/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.web.auth;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.dev33.satoken.secure.SaSecureUtil;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.StrUtil;
import com.yiring.auth.domain.user.User;
import com.yiring.auth.domain.user.UserRepository;
import com.yiring.auth.param.auth.LoginParam;
import com.yiring.auth.param.auth.RegisterParam;
import com.yiring.auth.param.auth.SafeParam;
import com.yiring.auth.util.Auths;
import com.yiring.auth.vo.auth.LoginVo;
import com.yiring.common.core.Result;
import com.yiring.common.exception.BusinessException;
import com.yiring.common.util.Commons;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.extensions.Extension;
import io.swagger.v3.oas.annotations.extensions.ExtensionProperty;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.data.domain.Example;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 用户认证控制器
 *
 * @author Jim
 * @version 0.1
 * 2022/1/24 14:13
 */

@Slf4j
@Validated
@Tag(
    name = "Auth",
    description = "身份认证",
    extensions = { @Extension(properties = { @ExtensionProperty(name = "x-order", value = "-9999") }) }
)
@RestController
@RequestMapping("/auth/")
@RequiredArgsConstructor
public class AuthController {

    final Auths auths;
    final UserRepository userRepository;

    @Operation(summary = "注册")
    @PostMapping(value = "register")
    public Result<String> register(@ParameterObject @Validated RegisterParam param) {
        // 检查用户名是否存在
        long count = userRepository.count(Example.of(User.builder().username(param.getUsername()).build()));
        if (count > 0) {
            throw BusinessException.i18n("Code.100000");
        }

        // 检查手机号是否存在
        count = userRepository.count(Example.of(User.builder().mobile(param.getMobile()).build()));
        if (count > 0) {
            throw BusinessException.i18n("Code.100001");
        }

        // 检查邮箱是否存在
        if (StrUtil.isNotBlank(param.getEmail())) {
            count = userRepository.count(Example.of(User.builder().email(param.getEmail()).build()));
            if (count > 0) {
                throw BusinessException.i18n("Code.100002");
            }
        }

        // 构建用户信息写入数据库
        User user = User
            .builder()
            .avatar(param.getAvatar())
            .mobile(param.getMobile())
            .realName(param.getRealName())
            .username(param.getUsername())
            .password(SaSecureUtil.sha256(param.getPassword()))
            .disabled(param.getDisabled())
            .build();
        userRepository.saveAndFlush(user);
        return Result.ok();
    }

    @Operation(summary = "登录")
    @PostMapping("login")
    public Result<LoginVo> login(@ParameterObject @Validated LoginParam param, HttpServletRequest request) {
        // 查询用户信息是否匹配
        User user = userRepository.findByAccount(param.getAccount());
        if (user == null) {
            throw BusinessException.i18n("Code.100003");
        }

        // 检查用户是否已被删除
        if (Boolean.TRUE.equals(user.getDeleted())) {
            throw BusinessException.i18n("Code.100004");
        }

        // 检查用户是否被允许登录
        if (Boolean.TRUE.equals(user.getDisabled())) {
            throw BusinessException.i18n("Code.100005");
        }

        // 检查密码
        String cps = SaSecureUtil.sha256(param.getPassword());
        if (!cps.equals(user.getPassword())) {
            throw BusinessException.i18n("Code.100003");
        }

        // 更新用户信息
        user.setLastLoginIp(Commons.getClientIpAddress(request));
        user.setLastLoginTime(LocalDateTime.now());
        userRepository.saveAndFlush(user);

        // 登录
        StpUtil.login(user.getId());

        // 构建用户所需信息
        LoginVo vo = LoginVo.builder().userId(user.getId()).token(StpUtil.getTokenValue()).build();
        return Result.ok(vo);
    }

    @Operation(summary = "检查登录")
    @GetMapping("valid")
    public Result<Boolean> valid() {
        return Result.ok(StpUtil.isLogin());
    }

    @Operation(summary = "登出")
    @GetMapping("logout")
    public Result<String> logout() {
        StpUtil.logout();
        return Result.ok();
    }

    /**
     * 二次安全校验，搭配 @SaCheckSafe 实现对关键数据不可逆操作前的二次确认
     * 默认安全时间: 120s
     *
     * @param param 用户密码
     * @link { <a href="https://sa-token.dev33.cn/doc.html#/up/safe-auth">...</a> }
     */
    @SaCheckLogin
    @Operation(summary = "安全验证")
    @GetMapping("safe")
    public Result<String> safe(@ParameterObject @Validated SafeParam param) {
        User user = auths.getLoginUser();
        if (SaSecureUtil.sha256(param.getPassword()).equals(user.getPassword())) {
            StpUtil.openSafe(360);
            return Result.ok();
        }

        return Result.no();
    }
}
