/* (C) 2022 YiRing, Inc. */
package com.yiring.common.web;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import com.yiring.common.annotation.DownloadResponse;
import com.yiring.common.core.Minio;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.DownloadParam;
import com.yiring.common.service.FileManageService;
import com.yiring.common.util.FileUtils;
import com.yiring.common.vo.ImageInfo;
import io.minio.GetObjectResponse;
import io.minio.StatObjectResponse;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.extensions.Extension;
import io.swagger.v3.oas.annotations.extensions.ExtensionProperty;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.NotBlank;
import java.sql.Timestamp;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * Minio S3
 *
 * @author Jim
 */

@Slf4j
@Validated
@Tag(
    name = "文件管理",
    description = "file",
    extensions = { @Extension(properties = { @ExtensionProperty(name = "x-order", value = "-9997") }) }
)
@RequiredArgsConstructor
@RestController
@RequestMapping("/common/file/")
public class MinioController {

    final Minio minio;
    final FileManageService fileManageService;

    /**
     * minio 上传文件，成功返回文件 url
     */
    @Operation(summary = "文件上传")
    @PostMapping(value = "upload", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public Result<String> upload(@Parameter(name = "文件", required = true) @RequestPart("file") MultipartFile file) {
        try {
            String link = fileManageService.upload(file);
            return Result.ok(link);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw Status.BAD_REQUEST.exception();
        }
    }

    @Operation(summary = "Base64 图片上传")
    @Parameter(
        name = "base64Image",
        description = "Base64 图片信息",
        example = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAQAAAALCAYAAABYpyyrAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAAFiUAABYlAUlSJPAAAAATSURBVBhXYzAwNP6PjIeKgPF/ABj+RUX4hZfVAAAAAElFTkSuQmCC",
        required = true,
        in = ParameterIn.QUERY
    )
    @PostMapping(value = "uploadBase64Image")
    public Result<String> uploadBase64Image(@NotBlank(message = "图片 Base64 信息不能为空") String base64Image) {
        // 解析 Base64 图片信息
        try (ImageInfo image = FileUtils.parseBase64ImageText(base64Image)) {
            // 获取文件信息以及默认存储地址
            String uuid = IdUtil.getSnowflakeNextIdStr();
            String object = minio.buildUploadPath(
                uuid,
                "." + image.getWidth() + "x" + image.getHeight() + "." + image.getSuffix()
            );

            // 上传原文件
            minio.putObject(image.getStream(), image.getContentType(), object);
            return Result.ok(minio.getDefaultURI(object));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw Status.BAD_REQUEST.exception();
        }
    }

    /**
     * MinIO 文件下载（非公开桶）
     *
     * @param response HttpServletResponse
     * @param param    请求参数
     */
    @DownloadResponse
    @Operation(summary = "文件下载")
    @GetMapping(value = "download", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public void download(HttpServletResponse response, @ParameterObject @Validated DownloadParam param) {
        try (GetObjectResponse object = minio.getObject(param.getBucket(), param.getObject())) {
            StatObjectResponse statObject = minio.objectStat(param.getBucket(), param.getObject());
            long lastModified = Timestamp.from(statObject.lastModified().toInstant()).getTime();
            FileUtils.download(
                response,
                object,
                statObject.size(),
                FileUtil.getName(statObject.object()),
                statObject.contentType(),
                lastModified
            );
        } catch (Exception e) {
            throw Status.BAD_REQUEST.exception(e.getMessage());
        }
    }
}
