/* (C) 2022 YiRing, Inc. */
package com.yiring.websocket.interceptor;

import com.yiring.auth.domain.user.User;
import com.yiring.auth.util.Auths;
import com.yiring.common.core.Redis;
import com.yiring.websocket.constant.RedisKey;
import com.yiring.websocket.domain.StompPrincipal;
import java.util.LinkedList;
import java.util.Map;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.simp.SimpMessageHeaderAccessor;
import org.springframework.messaging.simp.stomp.StompCommand;
import org.springframework.messaging.simp.stomp.StompHeaderAccessor;
import org.springframework.messaging.support.ChannelInterceptor;
import org.springframework.messaging.support.MessageHeaderAccessor;
import org.springframework.messaging.support.NativeMessageHeaderAccessor;
import org.springframework.stereotype.Component;

/**
 * ClientInboundChannelInterceptor
 * 接收客户端消息的拦截器
 *
 * @author ifzm
 * @version 0.1
 * 2019/9/28 20:58
 */

@Slf4j
@Component
@RequiredArgsConstructor
public class ClientInboundChannelInterceptor implements ChannelInterceptor {

    final Redis redis;
    final Auths auths;

    private final Object lock = new Object();

    @Override
    public Message<?> preSend(@NonNull Message<?> message, @NonNull MessageChannel channel) {
        StompHeaderAccessor accessor = MessageHeaderAccessor.getAccessor(message, StompHeaderAccessor.class);
        assert accessor != null;
        if (StompCommand.CONNECT.equals(accessor.getCommand())) {
            Object raw = message.getHeaders().get(NativeMessageHeaderAccessor.NATIVE_HEADERS);
            if (raw instanceof Map) {
                StompPrincipal principal = new StompPrincipal();
                principal.setSession(accessor.getSessionId());

                Object tokens = ((Map<?, ?>) raw).get("token");
                if (tokens instanceof LinkedList) {
                    String token = ((LinkedList<?>) tokens).getFirst().toString();
                    User user = auths.getUserByToken(token);
                    principal.setUser(user.getUsername());
                    principal.setType(StompPrincipal.Type.LOGIN_USER);
                } else {
                    principal.setUser("Guest." + principal.getSession());
                    principal.setType(StompPrincipal.Type.GUEST_USER);
                }

                accessor.setUser(principal);
                synchronized (lock) {
                    redis.hset(RedisKey.STOMP_ONLINE_USERS, principal.getSession(), principal);
                    log.info(
                        "STOMP Online Users: {} (incr: +1, session: {}, user: `{}`)",
                        redis.hsize(RedisKey.STOMP_ONLINE_USERS),
                        principal.getSession(),
                        principal.getUser()
                    );
                }
            }
        } else if (StompCommand.DISCONNECT.equals(accessor.getCommand())) {
            StompPrincipal principal = (StompPrincipal) accessor.getUser();
            if (principal != null && !message.getHeaders().containsKey(SimpMessageHeaderAccessor.HEART_BEAT_HEADER)) {
                synchronized (lock) {
                    redis.hdel(RedisKey.STOMP_ONLINE_USERS, principal.getSession());
                    log.info(
                        "STOMP Online Users: {} (incr: -1, session: {}, user: `{}`)",
                        redis.hsize(RedisKey.STOMP_ONLINE_USERS),
                        principal.getSession(),
                        principal.getUser()
                    );
                }
            }
        }

        return message;
    }
}
